#version 100

precision highp float;
//precision mediump float;

const int MAX_LIGHTS			= 1;
const float LIGHT_OMNI			= 0.0;
const float LIGHT_POINT			= 1.0;
const float LIGHT_DIRECTIONAL	= 2.0;

uniform vec4 color;
uniform bool useNormal;
uniform bool useTexture;
uniform sampler2D uSampler;
uniform vec2 material;

uniform vec3 backgroundColor;

uniform int lightEnable;
uniform int fogEnable;
uniform float fogDistance;

uniform vec3 lightSource0;
uniform vec4 lightFlags0;

varying vec3 normalInterp;
varying vec3 pureNormal;
varying vec3 vertPos;
varying vec2 vTextureCoord;
varying vec3 lightParams_pos;
varying float lightParams_flag0;
varying float lightParams_flag1;

const float shininess = 10.0;
const float screenGamma = 1.5;

void main(void)
{
	vec4 diffuse;
	if(useTexture)
	{
		diffuse = texture2D(uSampler, vec2(vTextureCoord.s, vTextureCoord.t));
	}
	else
	{
		diffuse = color;
	}
	if(diffuse[3] == 0.0)
	{
		discard;
	}
	
	if(lightEnable == 0)
	{
		gl_FragColor = diffuse;
	}
	else
	{
		vec4 finalColor;

		vec3 normal = normalize(normalInterp);
		float ambient = material[0];

		float flag0 = lightParams_flag0;
		vec3 lightPos = lightParams_pos;
		vec3 lightDir = normalize(lightPos - vertPos);
		float lambertian = max(dot(lightDir,normal), 0.0);

		float specular = 0.0;
		float attenuation = lightFlags0.z;

		if(flag0 == LIGHT_DIRECTIONAL || flag0 == LIGHT_POINT)
		{
			float b = lightParams_flag1;
			float dist = distance(lightPos,vertPos);
			float attenFrac = (1.0+b*dist*dist);
			if(attenFrac == 0.0)
				attenuation = 0.0;
			else
				attenuation = 1.0/attenFrac;
		}
		else
		{
			lambertian = max(0.3, lambertian);
		}
		lambertian *= attenuation;

		if(lambertian > 0.0) {
			vec3 viewDir = normalize(-vertPos);
			vec3 halfDir = normalize(lightDir + viewDir);
			float specAngle = max(dot(halfDir, normal), 0.0);
			specular = pow(specAngle, shininess)*attenuation;
		}

		vec3 colorLinear = ambient +
							 lambertian * vec3(diffuse) +
							 specular * material[1];
		vec3 colorGammaCorrected = pow(colorLinear, vec3(1.0/screenGamma));
		finalColor = vec4(colorGammaCorrected, 1.0);

		finalColor = clamp(finalColor,vec4(0),vec4(1));
		finalColor[3] =  diffuse[3];

		if(fogEnable == 1)
		{
			float depth = clamp((gl_FragCoord.z / gl_FragCoord.w)/(fogDistance/2.0),0.0,2.0);
			finalColor = mix(finalColor, vec4(backgroundColor,1.0), depth);
			float depth2 = 2.0-depth;
			finalColor[3] *= clamp(depth2,0.0,1.0);
		}
		gl_FragColor = finalColor;
	}
}
