#version 100
#extension GL_OES_texture_3D : enable 

precision highp float;
//precision mediump float;

uniform sampler3D Density;

uniform mat4 inverseModelMatrix;
uniform mat4 inverseViewMatrix;

uniform vec4 color;//##

uniform vec2 ufocalLength;
uniform vec2 screenSize;
uniform vec3 rayOrigin;
uniform vec3 lightPosition;
uniform float dayAmmount;
uniform float transparency;

const float maxDist = sqrt(2.0);
const float densityFactor = 6.0;
const int NumSamples = 16;
//##const int NumSamples = 256;

struct Ray {
    vec3 Origin;
    vec3 Dir;
};

struct AABB {
    vec3 Min;
    vec3 Max;
};

bool IntersectBox(Ray r, AABB aabb, out float t0, out float t1)
{
    vec3 invR = 1.0 / r.Dir;
    vec3 tbot = invR * (aabb.Min-r.Origin);
    vec3 ttop = invR * (aabb.Max-r.Origin);
    vec3 tmin = min(ttop, tbot);
    vec3 tmax = max(ttop, tbot);
    vec2 t = max(tmin.xx, tmin.yz);
    t0 = max(t.x, t.y);
    t = min(tmax.xx, tmax.yz);
    t1 = min(t.x, t.y);
    return t0 <= t1;
}

void main(void)
{
	gl_FragColor = color;

	float stepSize = maxDist/float(NumSamples);
	float lscale = 1.0;
	float Absorption = 1.0;
	//vec3 lightPosition = vec3(0.25,1.0,3);
	vec3 LightIntensity = vec3(10.0*dayAmmount);

    vec3 rayDirection;
	vec2 fragCoord = gl_FragCoord.xy;
	if(fragCoord.x > screenSize.x)
	{
		fragCoord.x -= screenSize.x;
	}
    rayDirection.xy = 2.0 * fragCoord / (screenSize) - 1.0;
    rayDirection.z = -ufocalLength.x;
	rayDirection.x *= screenSize.x/screenSize.y;
    rayDirection = (inverseViewMatrix * vec4(rayDirection, 0)).xyz;
	vec3 _rayOrigin = (inverseModelMatrix*vec4(rayOrigin,0)).xyz;

    Ray eye = Ray( _rayOrigin, rayDirection);
    AABB aabb = AABB(vec3(-1.0), vec3(+1.0));

    float tnear, tfar;
    IntersectBox(eye, aabb, tnear, tfar);
    if (tnear < 0.0) tnear = 0.0;

    vec3 rayStart = eye.Origin + eye.Dir * tnear;
    vec3 rayStop = eye.Origin + eye.Dir * tfar;
    rayStart = 0.5 * (rayStart + 1.0);
    rayStop = 0.5 * (rayStop + 1.0);

    vec3 pos = rayStart;
    vec3 step = normalize(rayStop-rayStart) * stepSize;
    float travel = distance(rayStop, rayStart);
    float T = 1.0;
    vec3 Lo = vec3(0.0);

    for (int i=0; i < NumSamples && travel > 0.0; ++i, pos += step, travel -= stepSize) {

        float density = texture3D(Density, pos).x * densityFactor;
        if (density <= 0.0)
            continue;

        T *= 1.0-density*stepSize*Absorption;
        if (T <= 0.01)
            break;

        vec3 lightDir = normalize(lightPosition-pos)*lscale;
        float Tl = 1.0;
        vec3 lpos = pos + lightDir;
        vec3 Li = LightIntensity*Tl;
        Lo += Li*T*density*stepSize;
    }
	gl_FragColor = transparency * vec4(Lo,1.0-T);
}
