#version 100

precision highp float;
//precision mediump float;

attribute vec3 aVertexPosition;
attribute vec3 aVertexNormal;
attribute vec2 aTextureCoord;

uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat4 projectionMatrix;
uniform mat4 transpose_inverse_mv;

uniform bool useNormal;
uniform bool useTexture;

uniform vec2 waterFlags;

uniform vec3 lightSource0;
uniform vec4 lightFlags0;

varying vec3 normalInterp;
varying vec3 pureNormal;
varying vec3 vertPos;
varying vec2 vTextureCoord;
varying vec3 lightParams_pos;
varying float lightParams_flag0;
varying float lightParams_flag1;

void main(void) {
	mat4 mv = viewMatrix * modelMatrix;
	vec4 vertexPosition = vec4(aVertexPosition,1.0);
	if(waterFlags[0] == 1.0)
		vertexPosition.y = sin(vertexPosition.x + waterFlags[1])*cos(vertexPosition.z+waterFlags[1]);

	gl_Position = projectionMatrix * mv * vertexPosition;

	vec4 vertPos4 = mv * vec4(aVertexPosition,1.0);
    vertPos = normalize(vec3(vertPos4));

	vTextureCoord = vec2(0,0);
	pureNormal = vec3(0,0,0);
	normalInterp = vec3(0,0,0);

	if(useNormal)
	{
		pureNormal = aVertexNormal;
		mat3 normalMat = mat3(transpose_inverse_mv);
		normalInterp = normalize(vec3(normalMat * aVertexNormal));
	}
	if(useTexture)
	{
		vTextureCoord = aTextureCoord;	
	}

	lightParams_pos = normalize(vec3(mat3(viewMatrix)*lightSource0));
	lightParams_flag0 = lightFlags0.x;
	lightParams_flag1 = lightFlags0.y;
}
